package cz.vsb.fei.efrei.lab04;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class ReflectedFighter {

	public static void main(String[] args) {
		List<Fighter> fighters = Stream
				.concat(Stream.generate(Utils::generateHeroName).map(Hero::new).limit(10),
						Stream.generate(Utils::generateMonsterName).map(Monster::new).limit(10))
				.collect(Collectors.toList());
		Collections.shuffle(fighters);

		for (Fighter fighter : fighters) {
			printUsingReflection(fighter);
		}

		for(Car car : Stream.generate(Car::generate).limit(10).toList()) {
			printUsingReflection(car);
		}
	}

	private static void printUsingReflection(Object object) {
		try {
			Class<?> fighterClass = object.getClass();
			Field[] fields = fighterClass.getDeclaredFields();
			List<Field> listOfFields = new ArrayList<>();
			listOfFields.addAll(List.of(fields));
			listOfFields.sort(Comparator.comparing(field -> {
				if (field.getAnnotation(Order.class) != null) {
					return field.getAnnotation(Order.class).value();
				}
				return 0;
			}));
			for (Field field : listOfFields) {
				if (Modifier.isStatic(field.getModifiers())) {
					continue;
				}
				Formatter f = field.getAnnotation(Formatter.class);
				field.setAccessible(true);
				int length = 10;
				String name = field.getName();
				boolean hidden = false;
				if (f != null) {
					length = f.length();
					name = f.name().isEmpty()?name:f.name();
					hidden = f.hidden();
				}
				if (hidden) {
					continue;
				}
				System.out.format("%s: %" + length + "s  ", name, field.get(object));
			}
			System.out.println();
		} catch (IllegalArgumentException | IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
